package gregtechmod.api.enums;

import gregtechmod.api.util.GT_Utility;

import java.util.ArrayList;

/**
 * This is some kind of Periodic Table, which I use to determine Properties of the Materials.
 */
public enum Element {
	_NULL	(   0,   0,   0,          -1, null		, ""				),
	H		(   1,   0,   0,          -1, null		, "Hydrogen"		),
	D		(   1,   1,   0,          -1, "H"		, "Deuterium"		),
	T		(   1,   2,   0,          -1, "D"		, "Tritium"			),
	He		(   2,   2,   0,          -1, null		, "Helium"			),
	He_3	(   2,   1,   0,          -1, "H&D"		, "Helium-3"		),
	Li		(   3,   4,   0,          -1, null		, "Lithium"			),
	Be		(   4,   5,   0,          -1, null		, "Beryllium"		),
	B		(   5,   5,   0,          -1, null		, "Boron"			),
	C		(   6,   6,   0,          -1, null		, "Carbon"			),
	N		(   7,   7,   0,          -1, null		, "Nitrogen"		),
	O		(   8,   8,   0,          -1, null		, "Oxygen"			),
	F		(   9,   9,   0,          -1, null		, "Fluorine"		),
	Ne		(  10,  10,   0,          -1, null		, "Neon"			),
	Na		(  11,  11,   0,          -1, null		, "Sodium"			),
	Mg		(  12,  12,   0,          -1, null		, "Magnesium"		),
	Al		(  13,  13,   0,          -1, null		, "Aluminium"		),
	Si		(  14,  14,   0,          -1, null		, "Silicon"			),
	P		(  15,  15,   0,          -1, null		, "Phosphorus"		),
	S		(  16,  16,   0,          -1, null		, "Sulfur"			),
	Cl		(  17,  18,   0,          -1, null		, "Chlorine"		),
	Ar		(  18,  22,   0,          -1, null		, "Argon"			),
	K		(  19,  20,   0,          -1, null		, "Potassium"		),
	Ca		(  20,  20,   0,          -1, null		, "Calcium"			),
	Sc		(  21,  24,   0,          -1, null		, "Scandium"		),
	Ti		(  22,  26,   0,          -1, null		, "Titanium"		),
	V		(  23,  28,   0,          -1, null		, "Vanadium"		),
	Cr		(  24,  28,   0,          -1, null		, "Chrome"			),
	Mn		(  25,  30,   0,          -1, null		, "Manganese"		),
	Fe		(  26,  30,   0,          -1, null		, "Iron"			),
	Co		(  27,  32,   0,          -1, null		, "Cobalt"			),
	Ni		(  28,  30,   0,          -1, null		, "Nickel"			),
	Cu		(  29,  34,   0,          -1, null		, "Copper"			),
	Zn		(  30,  35,   0,          -1, null		, "Zinc"			),
	Ga		(  31,  39,   0,          -1, null		, "Gallium"			),
	Ge		(  32,  40,   0,          -1, null		, "Germanium"		),
	As		(  33,  42,   0,          -1, null		, "Arsenic"			),
	Se		(  34,  45,   0,          -1, null		, "Selenium"		),
	Br		(  35,  45,   0,          -1, null		, "Bromine"			),
	Kr		(  36,  48,   0,          -1, null		, "Krypton"			),
	Rb		(  37,  48,   0,          -1, null		, "Rubidium"		),
	Sr		(  38,  49,   0,          -1, null		, "Strontium"		),
	Y		(  39,  50,   0,          -1, null		, "Yttrium"			),
	Zr		(  40,  51,   0,          -1, null		, "Zirconium"		),
	Nb		(  41,  53,   0,          -1, null		, "Niobium"			),
	Mo		(  42,  53,   0,          -1, null		, "Molybdenum"		),
	Tc		(  43,  55,   0,          -1, null		, "Technetium"		),
	Ru		(  44,  57,   0,          -1, null		, "Ruthenium"		),
	Rh		(  45,  58,   0,          -1, null		, "Rhodium"			),
	Pd		(  46,  60,   0,          -1, null		, "Palladium"		),
	Ag		(  47,  60,   0,          -1, null		, "Silver"			),
	Cd		(  48,  64,   0,          -1, null		, "Cadmium"			),
	In		(  49,  65,   0,          -1, null		, "Indium"			),
	Sn		(  50,  68,   0,          -1, null		, "Tin"				),
	Sb		(  51,  70,   0,          -1, null		, "Antimony"		),
	Te		(  52,  75,   0,          -1, null		, "Tellurium"		),
	I		(  53,  74,   0,          -1, null		, "Iodine"			),
	Xe		(  54,  77,   0,          -1, null		, "Xenon"			),
	Cs		(  55,  77,   0,          -1, null		, "Caesium"			),
	Ba		(  56,  81,   0,          -1, null		, "Barium"			),
	La		(  57,  81,   0,          -1, null		, "Lantanium"		),
	Ce		(  58,  82,   0,          -1, null		, "Cerium"			),
	Pr		(  59,  81,   0,          -1, null		, "Praseodymium"	),
	Nd		(  60,  84,   0,          -1, null		, "Neidymium"		),
	Pm		(  61,  83,   0,          -1, null		, "Promethium"		),
	Sm		(  62,  88,   0,          -1, null		, "Samarium"		),
	Eu		(  63,  88,   0,          -1, null		, "Europium"		),
	Gd		(  64,  93,   0,          -1, null		, "Gadolinium"		),
	Tb		(  65,  93,   0,          -1, null		, "Terbium"			),
	Dy		(  66,  96,   0,          -1, null		, "Dysprosium"		),
	Ho		(  67,  97,   0,          -1, null		, "Holmium"			),
	Er		(  68,  99,   0,          -1, null		, "Erbium"			),
	Tm		(  69,  99,   0,          -1, null		, "Thulium"			),
	Yb		(  70, 103,   0,          -1, null		, "Ytterbium"		),
	Lu		(  71, 103,   0,          -1, null		, "Lutetium"		),
	Hf		(  72, 106,   0,          -1, null		, "Hafnium"			),
	Ta		(  73, 107,   0,          -1, null		, "Tantalum"		),
	W		(  74, 109,   0,          -1, null		, "Wolframium"		),
	Re		(  75, 111,   0,          -1, null		, "Rhenium"			),
	Os		(  76, 114,   0,          -1, null		, "Osmium"			),
	Ir		(  77, 115,   0,          -1, null		, "Iridium"			),
	Pt		(  78, 117,   0,          -1, null		, "Platinum"		),
	Au		(  79, 117,   0,          -1, null		, "Gold"			),
	Hg		(  80, 120,   0,          -1, null		, "Mercury"			),
	Tl		(  81, 123,   0,          -1, null		, "Thallium"		),
	Pb		(  82, 125,   0,          -1, null		, "Lead"			),
	Bi		(  83, 125,   0,          -1, null		, "Bismuth"			),
	Po		(  84, 124,   0,          -1, null		, "Polonium"		),
	At		(  85, 124,   0,          -1, null		, "Astatine"		),
	Rn		(  86, 134,   0,          -1, null		, "Radon"			),
	Fr		(  87, 134,   0,          -1, null		, "Francium"		),
	Ra		(  88, 136,   0,          -1, null		, "Radium"			),
	Ac		(  89, 136,   0,          -1, null		, "Actinium"		),
	Th		(  90, 140,   0,          -1, null		, "Thorium"			),
	Pa		(  91, 138,   0,          -1, null		, "Protactinium"	),
	U		(  92, 146,   0,          -1, null		, "Uranium"			),
	U_235	(  92, 143,   0,          -1, null		, "Uranium-235"		),
	Np		(  93, 144,   0,          -1, null		, "Neptunium"		),
	Pu		(  94, 152,   0,          -1, null		, "Plutonium"		),
	Pu_241	(  94, 149,   0,          -1, null		, "Plutonium-241"	),
	Am		(  95, 150,   0,          -1, null		, "Americum"		),
	Cm		(  96, 153,   0,          -1, null		, "Curium"			),
	Bk		(  97, 152,   0,          -1, null		, "Berkelium"		),
	Cf		(  98, 153,   0,          -1, null		, "Californium"		),
	Es		(  99, 153,   0,          -1, null		, "Einsteinium"		),
	Fm		( 100, 157,   0,          -1, null		, "Fermium"			),
	Md		( 101, 157,   0,          -1, null		, "Mendelevium"		),
	No		( 102, 157,   0,          -1, null		, "Nobelium"		),
	Lr		( 103, 159,   0,          -1, null		, "Lawrencium"		),
	Rf		( 104, 161,   0,          -1, null		, "Rutherfordium"	),
	Db		( 105, 163,   0,          -1, null		, "Dubnium"			),
	Sg		( 106, 165,   0,          -1, null		, "Seaborgium"		),
	Bh		( 107, 163,   0,          -1, null		, "Bohrium"			),
	Hs		( 108, 169,   0,          -1, null		, "Hassium"			),
	Mt		( 109, 167,   0,          -1, null		, "Meitnerium"		),
	Ds		( 110, 171,   0,          -1, null		, "Darmstadtium"	),
	Rg		( 111, 169,   0,          -1, null		, "Roentgenium"		),
	Cn		( 112, 173,   0,          -1, null		, "Copernicium"		),
	Uut		( 113, 171,   0,          -1, null		, "Ununtrium"		),
	Fl		( 114, 175,   0,          -1, null		, "Flerovium"		),
	Uup		( 115, 173,   0,          -1, null		, "Ununpentium"		),
	Lv		( 116, 177,   0,          -1, null		, "Livermorium"		),
	Uus		( 117, 177,   0,          -1, null		, "Ununseptium"		),
	Uuo		( 118, 176,   0,          -1, null		, "Ununoctium"		),
	
	Ma		(   0,   0, 100,          -1, null		, "Magic"			),
	Nt		(0, 100000,   0,          -1, null		, "Neutronium"		),
	
	$H		(-  1,-  0,   0,          -1, null		, "Anti-Hydrogen"		),
	$D		(-  1,-  1,   0,          -1, "H"		, "Anti-Deuterium"		),
	$T		(-  1,-  2,   0,          -1, "D"		, "Anti-Tritium"		),
	$He		(-  2,-  2,   0,          -1, null		, "Anti-Helium"			),
	$He_3	(-  2,-  1,   0,          -1, "H&D"		, "Anti-Helium-3"		),
	$Li		(-  3,-  4,   0,          -1, null		, "Anti-Lithium"		),
	$Be		(-  4,-  5,   0,          -1, null		, "Anti-Beryllium"		),
	$B		(-  5,-  5,   0,          -1, null		, "Anti-Boron"			),
	$C		(-  6,-  6,   0,          -1, null		, "Anti-Carbon"			),
	$N		(-  7,-  7,   0,          -1, null		, "Anti-Nitrogen"		),
	$O		(-  8,-  8,   0,          -1, null		, "Anti-Oxygen"			),
	$F		(-  9,-  9,   0,          -1, null		, "Anti-Fluorine"		),
	$Ne		(- 10,- 10,   0,          -1, null		, "Anti-Neon"			),
	$Na		(- 11,- 11,   0,          -1, null		, "Anti-Sodium"			),
	$Mg		(- 12,- 12,   0,          -1, null		, "Anti-Magnesium"		),
	$Al		(- 13,- 13,   0,          -1, null		, "Anti-Aluminium"		),
	$Si		(- 14,- 14,   0,          -1, null		, "Anti-Silicon"		),
	$P		(- 15,- 15,   0,          -1, null		, "Anti-Phosphorus"		),
	$S		(- 16,- 16,   0,          -1, null		, "Anti-Sulfur"			),
	$Cl		(- 17,- 18,   0,          -1, null		, "Anti-Chlorine"		),
	$Ar		(- 18,- 22,   0,          -1, null		, "Anti-Argon"			),
	$K		(- 19,- 20,   0,          -1, null		, "Anti-Potassium"		),
	$Ca		(- 20,- 20,   0,          -1, null		, "Anti-Calcium"		),
	$Sc		(- 21,- 24,   0,          -1, null		, "Anti-Scandium"		),
	$Ti		(- 22,- 26,   0,          -1, null		, "Anti-Titanium"		),
	$V		(- 23,- 28,   0,          -1, null		, "Anti-Vanadium"		),
	$Cr		(- 24,- 28,   0,          -1, null		, "Anti-Chrome"			),
	$Mn		(- 25,- 30,   0,          -1, null		, "Anti-Manganese"		),
	$Fe		(- 26,- 30,   0,          -1, null		, "Anti-Iron"			),
	$Co		(- 27,- 32,   0,          -1, null		, "Anti-Cobalt"			),
	$Ni		(- 28,- 30,   0,          -1, null		, "Anti-Nickel"			),
	$Cu		(- 29,- 34,   0,          -1, null		, "Anti-Copper"			),
	$Zn		(- 30,- 35,   0,          -1, null		, "Anti-Zinc"			),
	$Ga		(- 31,- 39,   0,          -1, null		, "Anti-Gallium"		),
	$Ge		(- 32,- 40,   0,          -1, null		, "Anti-Germanium"		),
	$As		(- 33,- 42,   0,          -1, null		, "Anti-Arsenic"		),
	$Se		(- 34,- 45,   0,          -1, null		, "Anti-Selenium"		),
	$Br		(- 35,- 45,   0,          -1, null		, "Anti-Bromine"		),
	$Kr		(- 36,- 48,   0,          -1, null		, "Anti-Krypton"		),
	$Rb		(- 37,- 48,   0,          -1, null		, "Anti-Rubidium"		),
	$Sr		(- 38,- 49,   0,          -1, null		, "Anti-Strontium"		),
	$Y		(- 39,- 50,   0,          -1, null		, "Anti-Yttrium"		),
	$Zr		(- 40,- 51,   0,          -1, null		, "Anti-Zirconium"		),
	$Nb		(- 41,- 53,   0,          -1, null		, "Anti-Niobium"		),
	$Mo		(- 42,- 53,   0,          -1, null		, "Anti-Molybdenum"		),
	$Tc		(- 43,- 55,   0,          -1, null		, "Anti-Technetium"		),
	$Ru		(- 44,- 57,   0,          -1, null		, "Anti-Ruthenium"		),
	$Rh		(- 45,- 58,   0,          -1, null		, "Anti-Rhodium"		),
	$Pd		(- 46,- 60,   0,          -1, null		, "Anti-Palladium"		),
	$Ag		(- 47,- 60,   0,          -1, null		, "Anti-Silver"			),
	$Cd		(- 48,- 64,   0,          -1, null		, "Anti-Cadmium"		),
	$In		(- 49,- 65,   0,          -1, null		, "Anti-Indium"			),
	$Sn		(- 50,- 68,   0,          -1, null		, "Anti-Tin"			),
	$Sb		(- 51,- 70,   0,          -1, null		, "Anti-Antimony"		),
	$Te		(- 52,- 75,   0,          -1, null		, "Anti-Tellurium"		),
	$I		(- 53,- 74,   0,          -1, null		, "Anti-Iodine"			),
	$Xe		(- 54,- 77,   0,          -1, null		, "Anti-Xenon"			),
	$Cs		(- 55,- 77,   0,          -1, null		, "Anti-Caesium"		),
	$Ba		(- 56,- 81,   0,          -1, null		, "Anti-Barium"			),
	$La		(- 57,- 81,   0,          -1, null		, "Anti-Lantanium"		),
	$Ce		(- 58,- 82,   0,          -1, null		, "Anti-Cerium"			),
	$Pr		(- 59,- 81,   0,          -1, null		, "Anti-Praseodymium"	),
	$Nd		(- 60,- 84,   0,          -1, null		, "Anti-Neidymium"		),
	$Pm		(- 61,- 83,   0,          -1, null		, "Anti-Promethium"		),
	$Sm		(- 62,- 88,   0,          -1, null		, "Anti-Samarium"		),
	$Eu		(- 63,- 88,   0,          -1, null		, "Anti-Europium"		),
	$Gd		(- 64,- 93,   0,          -1, null		, "Anti-Gadolinium"		),
	$Tb		(- 65,- 93,   0,          -1, null		, "Anti-Terbium"		),
	$Dy		(- 66,- 96,   0,          -1, null		, "Anti-Dysprosium"		),
	$Ho		(- 67,- 97,   0,          -1, null		, "Anti-Holmium"		),
	$Er		(- 68,- 99,   0,          -1, null		, "Anti-Erbium"			),
	$Tm		(- 69,- 99,   0,          -1, null		, "Anti-Thulium"		),
	$Yb		(- 70,-103,   0,          -1, null		, "Anti-Ytterbium"		),
	$Lu		(- 71,-103,   0,          -1, null		, "Anti-Lutetium"		),
	$Hf		(- 72,-106,   0,          -1, null		, "Anti-Hafnium"		),
	$Ta		(- 73,-107,   0,          -1, null		, "Anti-Tantalum"		),
	$W		(- 74,-109,   0,          -1, null		, "Anti-Wolframium"		),
	$Re		(- 75,-111,   0,          -1, null		, "Anti-Rhenium"		),
	$Os		(- 76,-114,   0,          -1, null		, "Anti-Osmium"			),
	$Ir		(- 77,-115,   0,          -1, null		, "Anti-Iridium"		),
	$Pt		(- 78,-117,   0,          -1, null		, "Anti-Platinum"		),
	$Au		(- 79,-117,   0,          -1, null		, "Anti-Gold"			),
	$Hg		(- 80,-120,   0,          -1, null		, "Anti-Mercury"		),
	$Tl		(- 81,-123,   0,          -1, null		, "Anti-Thallium"		),
	$Pb		(- 82,-125,   0,          -1, null		, "Anti-Lead"			),
	$Bi		(- 83,-125,   0,          -1, null		, "Anti-Bismuth"		),
	$Po		(- 84,-124,   0,          -1, null		, "Anti-Polonium"		),
	$At		(- 85,-124,   0,          -1, null		, "Anti-Astatine"		),
	$Rn		(- 86,-134,   0,          -1, null		, "Anti-Radon"			),
	$Fr		(- 87,-134,   0,          -1, null		, "Anti-Francium"		),
	$Ra		(- 88,-136,   0,          -1, null		, "Anti-Radium"			),
	$Ac		(- 89,-136,   0,          -1, null		, "Anti-Actinium"		),
	$Th		(- 90,-140,   0,          -1, null		, "Anti-Thorium"		),
	$Pa		(- 91,-138,   0,          -1, null		, "Anti-Protactinium"	),
	$U		(- 92,-146,   0,          -1, null		, "Anti-Uranium"		),
	$U_235	(- 92,-143,   0,          -1, null		, "Anti-Uranium-235"	),
	$Np		(- 93,-144,   0,          -1, null		, "Anti-Neptunium"		),
	$Pu		(- 94,-152,   0,          -1, null		, "Anti-Plutonium"		),
	$Pu_241	(- 94,-149,   0,          -1, null		, "Anti-Plutonium-241"	),
	$Am		(- 95,-150,   0,          -1, null		, "Anti-Americum"		),
	$Cm		(- 96,-153,   0,          -1, null		, "Anti-Curium"			),
	$Bk		(- 97,-152,   0,          -1, null		, "Anti-Berkelium"		),
	$Cf		(- 98,-153,   0,          -1, null		, "Anti-Californium"	),
	$Es		(- 99,-153,   0,          -1, null		, "Anti-Einsteinium"	),
	$Fm		(-100,-157,   0,          -1, null		, "Anti-Fermium"		),
	$Md		(-101,-157,   0,          -1, null		, "Anti-Mendelevium"	),
	$No		(-102,-157,   0,          -1, null		, "Anti-Nobelium"		),
	$Lr		(-103,-159,   0,          -1, null		, "Anti-Lawrencium"		),
	$Rf		(-104,-161,   0,          -1, null		, "Anti-Rutherfordium"	),
	$Db		(-105,-163,   0,          -1, null		, "Anti-Dubnium"		),
	$Sg		(-106,-165,   0,          -1, null		, "Anti-Seaborgium"		),
	$Bh		(-107,-163,   0,          -1, null		, "Anti-Bohrium"		),
	$Hs		(-108,-169,   0,          -1, null		, "Anti-Hassium"		),
	$Mt		(-109,-167,   0,          -1, null		, "Anti-Meitnerium"		),
	$Ds		(-110,-171,   0,          -1, null		, "Anti-Darmstadtium"	),
	$Rg		(-111,-169,   0,          -1, null		, "Anti-Roentgenium"	),
	$Cn		(-112,-173,   0,          -1, null		, "Anti-Copernicium"	),
	$Uut	(-113,-171,   0,          -1, null		, "Anti-Ununtrium"		),
	$Fl		(-114,-175,   0,          -1, null		, "Anti-Flerovium"		),
	$Uup	(-115,-173,   0,          -1, null		, "Anti-Ununpentium"	),
	$Lv		(-116,-177,   0,          -1, null		, "Anti-Livermorium"	),
	$Uus	(-117,-177,   0,          -1, null		, "Anti-Ununseptium"	),
	$Uuo	(-118,-176,   0,          -1, null		, "Anti-Ununoctium"		),
	
	$Ma		(   0,   0,-100,          -1, null		, "Anti-Magic"			),
	$Nt		(0,-100000,   0,          -1, null		, "Anti-Neutronium"		);
	
	public static Element get(String aMaterialName) {
		Object tObject = GT_Utility.getFieldContent(Element.class, aMaterialName, false, false);
		if (tObject != null && tObject instanceof Element) return (Element)tObject;
		return _NULL;
	}
	
	public final short mProtons, mNeutrons, mAdditionalMass;
	public final int mHalfLifeSeconds;
	public final String mName, mDecayTo;
	
	/**
	 * Links to every pure Material containing just this Element.
	 */
	public ArrayList<Materials> mLinkedMaterials = new ArrayList<Materials>();
	
	/**
	 * @param aProtons Amount of Protons. Antiprotons if negative.
	 * @param aNeutrons Amount of Neutrons. Antineutrons if negative. (I could have made mistakes with the Neutron amount calculation, please tell me if I did something wrong)
	 * @param aHalfLifeSeconds Amount of Half Life this Material has in Seconds. -1 for stable Materials.
	 * @param aDecayTo String representing the Elements it decays to. Separated by an '&' Character.
	 * @param aName Name of the Element
	 */
	private Element(int aProtons, int aNeutrons, int aAdditionalMass, int aHalfLifeSeconds, String aDecayTo, String aName) {
		mProtons = (short)aProtons;
		mNeutrons = (short)aNeutrons;
		mAdditionalMass = (short)aAdditionalMass;
		mHalfLifeSeconds = aHalfLifeSeconds;
		mDecayTo = aDecayTo;
		mName = aName;
	}
	
	public int getProtons() {
		return mProtons;
	}
	
	public int getNeutrons() {
		return mNeutrons;
	}
	
	public int getMass() {
		return mProtons + mNeutrons + mAdditionalMass;
	}
	
	public static volatile int VERSION = 404;
}